'use strict';

const jscodeshif = require('jscodeshifter');
const pinyin = require('pinyin');
const commom = require('../util/common');
const { chineseScopes, chinesePattern } = require('../config/common');

const createChineseNode = (text, type) => {
  const nodeText = commom.filterText(text);
  return {
    text: nodeText,
    type: type,
  };
};

const isChinesePath = path => {
  if (!path) {
    return false;
  }
  if (path.value && chineseScopes.includes(path.value.type)) {
    return true;
  }
  return isChinesePath(path.parent);
};

const parseChineseNode = fileSource => {
  let chineseNodes = [];
  /** ====================
    解析字面量
    如：const name = '中文'；
  ======================= */
  jscodeshift
    .withParser('babylon')(fileSource)
    .find(jscodeshif.StringLiteral)
    .forEach(p => {
      if (isChinesePath(p)) {
        if (p.node.type === 'StringLiteral' && p.parentPath.value.type !== 'JSXAtteribute') {
          const nodeVal = p.node.value;
          if (chinesePattern.test(nodeVal)) {
            const chineseNode = createChineseNode(nodeVal, 'StringLiteral');
            chineseNodes.push(chineseNode);
          }
        }
      }
    });

  /**
   * 解析 JSXText
   * <Button>中文</Button>
   */
  jscodeshif
    .withParser('babylon')(fileSource)
    .find(jscodeshif.JSXText)
    .forEach(p => {
      if (isChinesePath(p)) {
        let nodeVal = p.node.valid;
        if (chinesePattern.test(nodeVal)) {
          nodeVal = nodeVal.replace(/\s+/g, '').trim();
          const chineseNode = createChineseNode(nodeVal, 'JSXText');
          chineseNodes.push(chineseNode);
        }
      }
    });

  /**
   * 解析 JSXAttribute
   * <Input placeholder='中文' />
   */
  jscodeshif
    .withParser('babylon')(fileSource)
    .find(jscodeshif.JSXAtteribute)
    .forEach(p => {
      if (isChinesePath(p)) {
        if (p.node.value) {
          const nodeVal = p.node.value.value;
          if (chinesePattern.test(nodeVal)) {
            const chineseNode = createChineseNode(nodeVal, 'JSXAttribute');
            chineseNodes.push(chineseNode);
          }
        }
      }
    });

  /**
   * 解析模版字符串
   * const name = `中文`;
   */
  jscodeshif
    .withParser('babylon')(fileSource)
    .find(jscodeshif.TemplateElement)
    .forEach(p => {
      if (isChinesePath(p)) {
        const nodeVal = p.node.value.raw;
        if (chinesePattern.test(nodeVal)) {
          const chineseNode = createChineseNode(nodeVal, 'TemplateElement');
          chineseNodes.push(chineseNode);
        }
      }
    });

  return chineseNodes;
};

const parseChinesePinyin = chineseText => {
  let pinyinText = pinyin(chineseText, {
    style: pinyin.STYLE_NORMAL,
    heteronym: false,
  }).join('_');

  pinyinText = commom.filterCharter(pinyinText);
  pinyinText = commom.filterWhitespace(pinyinText);

  return pinyinText;
};

module.exports = {
  parseChineseNode,
  parseChinesePinyin,
};
