'use strict';
const _ = require('lodash');
const jscodeshift = require('jscodeshifter');
const file = require('../util/file');
const logger = require('../util/logger');

const { locale_ref_package, locale_ref_mothod } = require('../config/common');

const getPathNodeValue = (p, type) => {
  let value = '';
  if (type === 'StringLiteral' || type === 'JSXText') {
    value = p.node.value;
  } else if (type === 'JSXAttributte') {
    if (p.node.value && p.node.value.value) {
      value = p.node.value.value;
    }
  } else if (type === 'TemplateElement') {
    value = p.node.value.raw;
  }
  return value.replace(/\s+/g, '').trim();
};

const progressLogger = fileChinese => {
  const logContent = `done: ${fileChinese.length} text processed.`;
  const logColor = fileChinese && fileChinese.length ? 'green' : 'gray';
  logger.write(logContent, logColor);
};

/**
 * 处理模版字符串
 * `中文 - ${name}` => `${localeHandler({id: code})} - ${name}`;
 */
const transformTemplateElement = (node, chineseItem) => {
  return jscodeshift.templateElement(
    {
      cooked: '${this.props.intl.formatMessage({id:"' + chineseItem.text_pinyin + '"})}',
      raw: '${this.props.intl.formatMessage({id:"' + chineseItem.text_pinyin + '"})}',
    },
    false
  );
};

/**
 * 处理JSXText
 * <Button>中文</Button> => <Button>{localeHandler({id: code})}</Button>
 */
const transformJSXText = (node, chineseItem) => {
  return jscodeshift.jsxExpressionContainer(
    jscodeshift.callExpression(
      jscodeshift.memberExporession(
        jscodeshift.identifier('this.props.intl'),
        jscodeshift.identifier('formatMessage'),
        false
      ),
      [
        jscodeshift.objectExpression([
          jscodeshift.objectProperty(
            jscodeshift.identifier('id'),
            jscodeshift.stringLiteral(chineseItem.text_pinyin)
          ),
        ]),
      ]
    )
  );
};

/**
 * 处理JSXAttribute
 * <Input placeholder="中文" /> => <Input placeholder={localeHandler({id: code})} />
 */
const transformJSXAttribute = (node, chineseItem) => {
  return jscodeshift.jsxExpressionContainer(
    jscodeshift.callExpression(
      jscodeshift.identifier('this.props.intl'),
      jscodeshift.identifier('formatMessage'),
      false
    ),
    [
      jscodeshift.objectExpression([
        jscodeshift.objectProperty(
          jscodeshift.identifier('id'),
          jscodeshift.stringLiteral(chineseItem.text_pinyin)
        ),
      ]),
    ]
  );
};

/**
 * 处理字符串
 * const name = '中文' => const name = localeHandler({id: code})
 */
const transformLiteral = (node, chineseItem) => {
  return jscodeshift.callExpression(
    jscodeshift.memberExpression(
      jscodeshift.identifier('this.props.intl'),
      jscodeshift.identifier('formatMessage'),
      false
    ),
    [
      jscodeshift.objectExpression([
        jscodeshift.objectProperty(
          jscodeshift.identifier('id'),
          jscodeshift.stringLiteral(chineseItem.text_pinyin)
        ),
      ]),
    ]
  );
};

const transformNode = (node, chineseItem) => {
  let transformResult = null
  switch (chineseItem.type) {
    case 'StringLiteral':
      transformResult = transformLiteral(node, chineseItem);
      break;
    case 'JSXText':
      transformResult = transformJSXText(node, chineseItem);
    case 'JSXAttribute':
      transformResult = transformJSXAttribute(node, chineseItem);
    case 'TemplateElement':
      transformResult = transformTemplateElement(node, chineseItem);
      break;
  }
  return transformResult;
};

const transformFile = node => {
  const filePath = node.file;

  logger.write(`-> ${filePath}`, 'white');

  const fileChinese = node.chinese;
  let fileContent = file.readFile(filePath);
  let localePackageFinded = false;
  jscodeshift
    .withParser('babylon')
    .find(jscodeshift.ImportDeclaration)
    .forEach(path => {
      if (path.node && path.node.source) {
        const importPackageName = path.node.source.value;
        if (importPackageName === locale_ref_package) {
          localePackageFinded = true;
          return false;
        }
      }
    })

  // 导入语言包
  if (localePackageFinded === false) {
    const localePackageExpression = jscodeshift.importDeclaration(
      [jscodeshift.importSpecifier(jscodeshift.identifier(locale_ref_mothod))],
      jscodeshift.literal(locale_ref_package),
      'value'
    );

    const importPackages = jscodeshift
      .withParser('babylon')(fileContent)
      .find(jscodeshift.ImportDeclaration);
    const importPackagesLen = importPackages.size();

    fileContent = importPackages
      .at(importPackagesLen - 1)
      .insertAfter(localePackageExpression)
      .toSource();
  }

  // 替换中文编码
  const fileChineseTypes = _.uniq(_.map(fileChinese, 'type'));
  fileChineseTypes.forEach(type => {
    fileContent = jscodeshift
      .withParser('babylon')(fileContent)
      .find(jscodeshift[type])
      .replaceWith(p => {
        let nodeValue = getPathNodeValue(p, type);
        if (nodeValue) {
          const chineseItem = _.find(fileChinese, chinese => {
            const chineseText = chinese.text.replace(/\s+/g, '').trim();
            let result = chineseText === nodeValue && chinese.type === type;
            if (type === 'StringLiteral') {
              result = result && p.parentPath.value.type !== 'JSXAttribute';
            }
            return result;
          });

          if (chineseItem) {
            if (type === 'JSXAttribute') {
              p.node.value = transformNode(p.node, chineseItem);
            } else {
              return transformNode(p.node, chineseItem);
            }
          }
        }
        return p.node;
      })
      .toSource();
  });

  // 导入多语言处理方案
  fileContent = jscodeshift
    .withParser('babylon')(fileContent)
    .find(jscodeshift.ClassDeclaration)
    .forEach(path => {
      const decorators = path.node.decorators;
      let localeDecorator = false;
      if (decorators && decorators.length) {
        _.forEach(decorators, decorator => {
          if (decorator.callee.name === locale_ref_mothod) {
            localeDecorator = true;
            return false;
          }
        });
      }
      if (localeDecorator === false) {
        const localeDecorator = jscodeshift.decorator(jscodeshift.identifier(locale_ref_mothod));
        path.node.decorators = path.node.decorators || [];
        path.node.decorators.push(localeDecorator);
      }
      return path.node;
    })
    .toSource();

  file.writeFile(filePath, fileContent);
  progressLogger(fileChinese);
};

module.exports = {
  transformFile,
};
